@@ -20,6 +20,7 @@ from dominate.dom_tag import dom_tag
 from glom import glom
 from lxml import etree
 from lxml.builder import ElementMaker
+from micawber import ProviderException, bootstrap_basic, cache
 from pandas.io.formats.style import Styler
 
 from datapane.client import DPError
@@ -130,6 +131,7 @@ class BaseElement(ABC):
 Block = t.Union["Group", "Select", "DataBlock"]
 BlockOrPrimitive = t.Union[Block, t.Any]  # TODO - expand
 PageOrPrimitive = t.Union["Page", BlockOrPrimitive]
+BlockList = t.List[Block]
 
 
 class SelectType(enum.Enum):
@@ -158,7 +160,7 @@ class LayoutBlock(BaseElement):
      - represents a subtree in the document
     """
 
-    blocks: t.List[Block] = None
+    blocks: BlockList = None
 
     def __init__(self, *arg_blocks: BlockOrPrimitive, blocks: t.List[BlockOrPrimitive] = None, **kwargs):
         self.blocks = blocks or list(arg_blocks)
@@ -296,8 +298,6 @@ class Group(LayoutBlock):
           `dp.Group(plot, table, columns=2)` or `dp.Group(blocks=[plot, table], columns=2)`
         """
         super().__init__(*arg_blocks, blocks=blocks, id=id, label=label)
-        # wrap str in Markdown block
-        self.blocks = [(Text(b) if isinstance(b, str) else b) for b in self.blocks]
 
         # set row/column handling
         if rows == 1 and columns == 1:
@@ -413,6 +413,7 @@ class Code(EmbeddedTextBlock):
             code: The source code
             language: The language of the code, most common languages are supported (optional - defaults to Python)
             id: A unique id for the block to aid querying (optional)
+            label: A label used when displaying the block (optional)
         """
         super().__init__(id=id, label=label, language=language)
         self.content = code
@@ -430,11 +431,47 @@ class HTML(EmbeddedTextBlock):
         Args:
             html: The HTML fragment to embed - can be a string or a [dominate](https://github.com/Knio/dominate/) tag
             id: A unique id for the block to aid querying (optional)
+            label: A label used when displaying the block (optional)
         """
         super().__init__(id=id, label=label)
         self.content = str(html)
 
 
+class Embed(EmbeddedTextBlock):
+    """
+    Embed blocks allow HTML from OEmbed providers (e.g. Youtube, Twitter, Vimeo) to be embedded in a report.
+    """
+
+    _tag = "Embed"
+    providers = bootstrap_basic(cache=cache.Cache())
+
+    def __init__(self, url: str, width: int = 640, height: int = 480, id: str = None, label: str = None):
+        """
+        Args:
+            url: The URL of the resource to be embedded
+            width: The width of the embedded object (optional)
+            height: The height of the embedded object (optional)
+            id: A unique id for the block to aid querying (optional)
+            label: A label used when displaying the block (optional)
+        """
+
+        try:
+            result = self.providers.request(url, maxwidth=width, maxheight=height)
+        except ProviderException:
+            raise DPError(f"No embed provider found for URL '{url}'")
+        super().__init__(
+            id=id,
+            label=label,
+            url=url,
+            title=result.get("title", "Title"),
+            provider_name=result.get("provider_name", "Embedding"),
+        )
+
+        # if "html" not in result:
+        #     raise DPError(f"Can't embed result from provider for URL '{url}'")
+        self.content = result["html"]
+
+
 NumberValue = t.Union[str, int, float]
 
 
